// Classification using NB with unknown components
// - off-line estimation
// - normal models
// -------------------------------------
clc, clear, close(winsid()), mode(0)
// =====================================
function f=normpdf(x,m,r)
  // density of normal distribution at x
  f=1/sqrt(2*%pi*r)*exp(-.5*(x-m)^2/(2*r));
endfunction
function [f,s]=hst(y,n)
  // normalized histogram of y
  // n    maximum of y (min=1)
  // f    probabilities
  // s values
  if argn(2)<2, n=max(y); end
  f=zeros(1,n);
  for i=1:length(y)
    f(y(i))=f(y(i))+1;
  end
  f=f/sum(f);
  s=1:n;
endfunction
// =====================================

nd=100;
// SIMULATION
kS=[-1 1];
for t=1:nd
  for i=1:2
    x(i,t)=rand(1,1,'n')+kS(i);
  end
  if sum(x(:,t))>0, y(t)=1; else y(t)=2; end
end  

// ESTIMATION
for j=1:2
  for i=1:2
    s=find(y==j);                  // find indexes of xi where y=j
    xc{i,j}=x(i,s);                // set of xi where y=j (clusters)
    mxy(i,j)=mean(xc{i,j},2);      // prob.fc. in clusters
  end
end
fy=hst(y,2);                       // f(y)

// CLASSIFICATION
for t=1:nd
  for j=1:2
    fyc(j)=fy(j);                  //f(xi|y=j)
    for i=1:2
      fyc(j)=fyc(j)*normpdf(x(i,t),mxy(i,j),.1);  // proximity (prod. in xi)
    end
  end
  py=fyc/sum(fyc);                  // f(y|x) - weights
  [nill,yp(t)]=max(py);             // argument maxima w - yp
end

// ACCURACY of classification
acc=sum(y==yp)/nd
