// Classification using NB with unknown components
// - off-line estimation
// - categorical models
// -------------------------------------
clc, clear, close(winsid()), mode(0)
// =====================================
function [f,s]=hst(y,n)
  // normalized histogram of y
  // n    maximum of y (min=1)
  // f    probabilities
  // s values
  if argn(2)<2, n=max(y); end
  f=zeros(1,n);
  for i=1:length(y)
    f(y(i))=f(y(i))+1;
  end
  f=f/sum(f);
  s=1:n;
endfunction
// =====================================

// SIMULATION
nd=100;
fxS=[.3 .7;                        // [f(x1)
     .8 .2];                       //  f(x2)]
fyS=[.02 .9 .05 .9               // [f(y=1|x1,x2)
     .98 .1 .95 .1];             //  f(y=2|x1,x2)]

for t=1:nd
  // simulation
  for i=1:2
    x(i,t)=sum(cumsum(fxS(i,:))<rand(1,1,'u'))+1;
  end
  k=2*(x(1,t)-1)+x(2,t);
  y(t)=sum(cumsum(fyS(:,k))<rand(1,1,'u'))+1;
end  

// ESTIMATION
for j=1:2
  for i=1:2
    s=find(y==j);                  // find indexes of xi where y=j
    xc{i,j}=x(i,s);                // set of xi where y=j (clusters)
    fxy{i,j}=hst(xc{i,j},2);       // prob.fc. in clusters
  end
end
fy=hst(y,2);                       // f(y)

// CLASSIFICATION
for t=1:nd
  for j=1:2
    fyc(j)=1;                      // f(xi|y=j)
    for i=1:2
      fyc(j)=fyc(j)*fxy{i,j}(x(i,t));  // product in xi
    end
  end
  py=fyc/sum(fyc);                  // f(y|x) - weights
  wt(:,t)=py;                       // remember weights
  [nill,yp(t)]=max(py);             // argument maxima py - yp
end

// ACCURACY of classification
acc=sum(y==yp)/nd
