// Classification based on mixture estimation
// - on-line estimation
// - discrete models  f(x1,x2|c)
// - initial data
// -------------------------------------
clc, clear, close(winsid()), mode(0)                                  // 1
getd();                                                               // 2
                                                                      // 3
nd=300; ni=100;                                                       // 4
// SIMULATION                                                         // 5
pS=[.4 .6];                                                           // 6
//      c =1 =2                                                       // 7
thS(1).x=[.03 .98      // x1                                          // 8
          .97 .02];                                                   // 9
thS(2).x=[.99 .03      // x2                                          // 10
          .01 .97];                                                   // 11
for t=1:(ni+nd)                                                       // 12
  c(t)=sum(cumsum(pS)<rand(1,1,'u'))+1;                               // 13
  for i=1:2                                                           // 14
    x(i,t)=sum(cumsum(thS(i).x(:,c(t)))<rand(1,1,'u'))+1;             // 15
  end                                                                 // 16
end                                                                   // 17
                                                                      // 18
// ESTIMATION                                                         // 19
xi=x(:,(nd+1):(nd+ni));                                               // 20
ci=c((nd+1):(nd+ni));                                                 // 21
select 1       // 1=without, 2=with  the knowledge of c               // 22
case 1                                                                // 23
  T=tab(xi);                                                          // 24
  [nill,n1]=max(T);                                                   // 25
  T(n1(1),n1(2))=0;                                                   // 26
  t1=rand(2,2,'u');                                                   // 27
  t1(n1(1),n1(2))=15;                                                 // 28
  t1=fnorm(t1,1);                                                     // 29
  [nill,n2]=max(T);                                                   // 30
  t2=rand(2,2,'u');                                                   // 31
  t2(n2(1),n2(2))=15;                                                 // 32
  t2=fnorm(t2,1);                                                     // 33
case 2                                                                // 34
  t1=zeros(2,2);                                                      // 35
  s1=find(ci==1);                                                     // 36
  t1=fnorm(tab(xi(:,s1),2,2)+1e-6,1);                                 // 37
  t2=zeros(2,2);                                                      // 38
  s2=find(ci==2);                                                     // 39
  t2=fnorm(tab(xi(:,s2),2,2)+1e-6,1);                                 // 40
end                                                                   // 41
thI=[pmult(t1(:,1)',t2(:,1)')'..                                      // 42
    pmult(t1(:,2)',t2(:,2)')'];                                       // 43
th=thI;                                                               // 44
ka=[1 1];                                    // prior counter         // 45
S=th.*(ones(4,1)*ka);                                                 // 46
                                                                      // 47
// time loop                                                          // 48
for t=1:nd                                                            // 49
  for j=1:2                                                           // 50
    k=2*(x(1,t)-1)+x(2,t);                                            // 51
    q(j)=th(k,j); // proximity = fj(x=xt)                             // 52
  end                                                                 // 53
  w=q/sum(q);                                // weights               // 54
  wt(:,t)=w;                                                          // 55
  for j=1:2                                                           // 56
    k=2*(x(1,t)-1)+x(2,t);                                            // 57
    S(k,j)=S(k,j)+w(j);                      // statistics            // 58
    ka(j)=ka(j)+w(j);                        //  update               // 59
    th(:,j)=fnorm(S(:,j)/sum(S(:,j)));                                // 60
  end                                                                 // 61
  th1(:,t)=th(:,1); th2(:,t)=th(:,2);                                 // 62
  [nill,cp(t)]=max(w);                       // CLASSIFICATION        // 63
end                                                                   // 64
                                                                      // 65
// ACCURACY of classification                                         // 66
disp 'Accuracy'                                                       // 67
s=1:nd;                                                               // 68
c=c(s); cp=cp(s);                                                     // 69
h=c2c(c,cp);                                 // reordering of comps   // 70
acc=sum(c==h(cp))/nd                                                  // 71
disp 'Simulated parametrs'                                            // 72
thI                                                                   // 73
disp 'Parameter estimates'                                            // 74
thE=fix(th*10000)/10000                                               // 75
                                                                      // 76
set(scf(),'position',[800 20 600 600]);                               // 77
subplot(211), plot(th1')                                              // 78
title 'Evolution of the estimates - first component'                  // 79
set(gca(),'data_bounds',[1 nd -.1 1.1])                               // 80
subplot(212), plot(th2')                                              // 81
title 'Evolution of the estimates - second component'                 // 82
set(gca(),'data_bounds',[1 nd -.1 1.1])                               // 83
