// Classification based on mixture estimation
// - on-line estimation
// - discrete models  
// - independent expl. vars f(x1,x2|c)=f(x1|c)f(x2|c)
// - initial data
// -------------------------------------
clc, clear, close(winsid()), mode(0)                                  // 1
getd();                                                               // 2
                                                                      // 3
ni=100; nd=300;     // munb. of initial and measured data             // 4
// SIMULATION                                                         // 5
pS=[.4 .6];                                                           // 6
//      c =1 =2                                                       // 7
thS(1).x=[.03 .98      // x1                                          // 8
          .97 .02];                                                   // 9
thS(2).x=[.99 .03      // x2                                          // 10
          .01 .97];                                                   // 11
for t=1:(ni+nd)                                                       // 12
  c(t)=sum(cumsum(pS)<rand(1,1,'u'))+1;                               // 13
  for i=1:2                                                           // 14
    x(i,t)=sum(cumsum(thS(i).x(:,c(t)))<rand(1,1,'u'))+1;             // 15
  end                                                                 // 16
end                                                                   // 17
                                                                      // 18
// ESTIMATION                                                         // 19
xi=x(:,(nd+1):(nd+ni));                                               // 20
ci=c((nd+1):(nd+ni));                                                 // 21
fx=list(); S=list();                                                  // 22
select 1       // 1=without, 2=with  the knowledge of c               // 23
case 1                                                                // 24
  T=tab(xi);                                                          // 25
  [nill,n1]=max(T);                                                   // 26
  T(n1(1),n1(2))=0;                                                   // 27
  fx(1)=rand(2,2,'u'); fx(1)(n1(1),n1(2))=15;                         // 28
  fx(1)=fnorm(fx(1),1);                                               // 29
  [nill,n2]=max(T);                                                   // 30
  fx(2)=rand(2,2,'u'); fx(2)(n2(1),n2(2))=15;                         // 31
  fx(2)=fnorm(fx(2),1);                                               // 32
case 2                                                                // 33
  fx(1)=zeros(2,2);                                                   // 34
  s1=find(ci==1);                                                     // 35
  fx(1)=fnorm(tab(xi(:,s1),2,2)+1e-6,1);                              // 36
  fx(2)=zeros(2,2);                                                   // 37
  s2=find(ci==2);                                                     // 38
  fx(2)=fnorm(tab(xi(:,s2),2,2)+1e-6,1);                              // 39
end                                                                   // 40
ka=[1 1];                                         // prior counter    // 41
S(1)=fx(1)*ka(1); S(2)=fx(2)*ka(2);                                   // 42
                                                                      // 43
// time loop                                                          // 44
for t=1:nd                                                            // 45
  for j=1:2                                                           // 46
    q(j)=1;                                                           // 47
    for i=1:2                                                         // 48
      q(j)=q(j)*fx(i)(x(i,t),j); // proximity = fj(xi=xti)            // 49
    end                                                               // 50
  end                                                                 // 51
  w=q/sum(q);                                     // weights          // 52
  wt(:,t)=w;                                                          // 53
  for j=1:2                                                           // 54
    for i=1:2                                                         // 55
      S(i)(x(i,t),j)=S(i)(x(i,t),j)+w(j);         // statistics       // 56
      ka(j)=ka(j)+w(j);                           //  update          // 57
      fx(i)(:,j)=fnorm(S(i)(:,j)/sum(S(i)(:,j))); // scalar estimates // 58
    end                                                               // 59
  end                                                                 // 60
  fx1t(:,t)=fx(1)(:,1);                           // joint            // 61
  fx2t(:,t)=fx(2)(:,1);                           //   estimates      // 62
  [nill,cp(t)]=max(w);   // CLASSIFICATION                            // 63
end                                                                   // 64
                                                                      // 65
// ACCURACY of classification                                         // 66
disp 'Accuracy'                                                       // 67
s=1:nd;                                                               // 68
c=c(s); cp=cp(s);                                                     // 69
h=c2c(c,cp);                                 // reordering of comps   // 70
acc=sum(c==h(cp))/nd                                                  // 71
                                                                      // 72
disp 'Simulated parametrs'                                            // 73
thS(1).x, thS(2).x                                                    // 74
disp 'Parameter estimates'                                            // 75
fix(fx(1)*100)/100                                                    // 76
fix(fx(2)*100)/100                                                    // 77
                                                                      // 78
set(scf(),'position',[800 20 600 600]);                               // 79
subplot(211), plot(fx1t')                                             // 80
title 'Evolution of the estimates - first component'                  // 81
set(gca(),'data_bounds',[1 nd -.1 1.1])                               // 82
subplot(212), plot(fx2t')                                             // 83
title 'Evolution of the estimates - second component'                 // 84
set(gca(),'data_bounds',[1 nd -.1 1.1])                               // 85
